@@ -1,6 +1,6 @@
 # Business::Shipping::Util
 # 
-# $Id: Util.pm,v 1.1 2003/12/22 03:49:05 db-ship Exp $
+# $Id: Util.pm,v 1.2 2004/01/03 03:11:20 db-ship Exp $
 # 
 # Copyright (c) 2003 Kavod Technologies, Dan Browning. All rights reserved. 
 # 
@@ -13,7 +13,7 @@ use strict;
 use warnings;
 
 use vars qw( $VERSION @EXPORT );
-$VERSION = do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 use base ( 'Exporter', 'Business::Shipping' );
 
 use Data::Dumper;
@@ -182,3 +182,15 @@ sub readfile
 	return $contents;
 }
 
+sub element_e_in_array_a
+{
+	my ( $e, @a ) = @_;
+	return unless $e and @a;
+	
+	for ( @a ) {
+		return 1 if $_ eq $e;
+	}
+	
+	return 0;
+}
+
